@@ -19,4 +19,6 @@ cargo build --release --target=x86_64-unknown-linux-gnu --locked
 cp -p ./target/x86_64-unknown-linux-gnu/release/arp-scan ./builds/arp-scan-v$CLI_VERSION-x86_64-unknown-linux-glibc
 ./builds/arp-scan-v$CLI_VERSION-x86_64-unknown-linux-glibc --version
 
-echo "Update the README instructions for v$CLI_VERSION"
\ No newline at end of file
+echo "Update the README instructions for v$CLI_VERSION"
+echo " ✓ Publish on crates.io"
+echo " ✓ Release on Github with Git tag v$CLI_VERSION"
\ No newline at end of file
